@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 # Advice: use repr(our_file.read()) to print the full output of tqdm
 # (else '\r' will replace the previous lines and you'll see only the latest.
-from __future__ import print_function
-
 import csv
 import os
 import re
@@ -31,23 +29,12 @@ class DeprecationError(Exception):
 
 
 # Ensure we can use `with closing(...) as ... :` syntax
-if getattr(StringIO, '__exit__', False) and \
-   getattr(StringIO, '__enter__', False):
+if getattr(StringIO, '__exit__', False) and getattr(StringIO, '__enter__', False):
     def closing(arg):
         return arg
 else:
     from contextlib import closing
 
-try:
-    _range = xrange
-except NameError:
-    _range = range
-
-try:
-    _unicode = unicode
-except NameError:
-    _unicode = str
-
 nt_and_no_colorama = False
 if os.name == 'nt':
     try:
@@ -135,14 +122,12 @@ class UnicodeIO(IOBase):
         return self.cursor
 
     def write(self, s):
-        self.text = self.text[:self.cursor] + s + \
-            self.text[self.cursor + len(s):]
+        self.text = self.text[:self.cursor] + s + self.text[self.cursor + len(s):]
         self.cursor += len(s)
 
     def read(self, n=-1):
         _cur = self.cursor
-        self.cursor = len(self) if n < 0 \
-            else min(_cur + n, len(self))
+        self.cursor = len(self) if n < 0 else min(_cur + n, len(self))
         return self.text[_cur:self.cursor]
 
     def getvalue(self):
@@ -163,42 +148,28 @@ def progressbar_rate(bar_str):
 
 def squash_ctrlchars(s):
     """Apply control characters in a string just like a terminal display"""
-    # Init variables
-    curline = 0  # current line in our fake terminal
-    lines = ['']  # state of our fake terminal
-
-    # Split input string by control codes
-    s_split = RE_ctrlchr.split(s)
-    s_split = filter(None, s_split)  # filter out empty splits
-
-    # For each control character or message
-    for nextctrl in s_split:
-        # If it's a control character, apply it
+    curline = 0
+    lines = ['']  # state of fake terminal
+    for nextctrl in filter(None, RE_ctrlchr.split(s)):
+        # apply control chars
         if nextctrl == '\r':
-            # Carriage return
-            # Go to the beginning of the line
-            # simplified here: we just empty the string
+            # go to line beginning (simplified here: just empty the string)
             lines[curline] = ''
         elif nextctrl == '\n':
-            # Newline
-            # Go to the next line
-            if curline < (len(lines) - 1):
-                # If already exists, just move cursor
-                curline += 1
-            else:
-                # Else the new line is created
+            if curline >= len(lines) - 1:
+                # wrap-around creates newline
                 lines.append('')
-                curline += 1
+            # move cursor down
+            curline += 1
         elif nextctrl == '\x1b[A':
-            # Move cursor up
+            # move cursor up
             if curline > 0:
                 curline -= 1
             else:
-                raise ValueError("Cannot go up, anymore!")
-        # Else, it is a message, we print it on current line
+                raise ValueError("Cannot go further up")
         else:
+            # print message on current line
             lines[curline] += nextctrl
-
     return lines
 
 
@@ -218,6 +189,8 @@ def test_format_num():
     assert float(format_num(1337)) == 1337
     assert format_num(int(1e6)) == '1e+6'
     assert format_num(1239876) == '1' '239' '876'
+    assert format_num(0.00001234) == '1.23e-5'
+    assert format_num(-0.1234) == '-0.123'
 
 
 def test_format_meter():
@@ -229,94 +202,76 @@ def test_format_meter():
 
     format_meter = tqdm.format_meter
 
-    assert format_meter(0, 1000, 13) == \
-        "  0%|          | 0/1000 [00:13<?, ?it/s]"
+    assert format_meter(0, 1000, 13) == "  0%|          | 0/1000 [00:13<?, ?it/s]"
     # If not implementing any changes to _tqdm.py, set prefix='desc'
     # or else ": : " will be in output, so assertion should change
-    assert format_meter(0, 1000, 13, ncols=68, prefix='desc: ') == \
-        "desc:   0%|                                | 0/1000 [00:13<?, ?it/s]"
-    assert format_meter(231, 1000, 392) == \
-        " 23%|" + unich(0x2588) * 2 + unich(0x258e) + \
-        "       | 231/1000 [06:32<21:44,  1.70s/it]"
-    assert format_meter(10000, 1000, 13) == \
-        "10000it [00:13, 769.23it/s]"
-    assert format_meter(231, 1000, 392, ncols=56, ascii=True) == \
-        " 23%|" + '#' * 3 + '6' + \
-        "            | 231/1000 [06:32<21:44,  1.70s/it]"
-    assert format_meter(100000, 1000, 13, unit_scale=True, unit='iB') == \
-        "100kiB [00:13, 7.69kiB/s]"
-    assert format_meter(100, 1000, 12, ncols=0, rate=7.33) == \
-        " 10% 100/1000 [00:12<02:02,  7.33it/s]"
+    assert format_meter(0, 1000, 13, ncols=68, prefix='desc: ') == (
+        "desc:   0%|                                | 0/1000 [00:13<?, ?it/s]")
+    assert format_meter(231, 1000, 392) == (" 23%|" + unich(0x2588) * 2 + unich(0x258e) +
+                                            "       | 231/1000 [06:32<21:44,  1.70s/it]")
+    assert format_meter(10000, 1000, 13) == "10000it [00:13, 769.23it/s]"
+    assert format_meter(231, 1000, 392, ncols=56, ascii=True) == " 23%|" + '#' * 3 + '6' + (
+        "            | 231/1000 [06:32<21:44,  1.70s/it]")
+    assert format_meter(100000, 1000, 13, unit_scale=True,
+                        unit='iB') == "100kiB [00:13, 7.69kiB/s]"
+    assert format_meter(100, 1000, 12, ncols=0,
+                        rate=7.33) == " 10% 100/1000 [00:12<02:02,  7.33it/s]"
     # ncols is small, l_bar is too large
     # l_bar gets chopped
     # no bar
     # no r_bar
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=10,
-            bar_format="************{bar:10}$$$$$$$$$$") == \
-        "**********"  # 10/12 stars since ncols is 10
+    # 10/12 stars since ncols is 10
+    assert format_meter(
+        0, 1000, 13, ncols=10,
+        bar_format="************{bar:10}$$$$$$$$$$") == "**********"
     # n_cols allows for l_bar and some of bar
     # l_bar displays
     # bar gets chopped
     # no r_bar
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=20,
-            bar_format="************{bar:10}$$$$$$$$$$") == \
-        "************        "  # all 12 stars and 8/10 bar parts
+    # all 12 stars and 8/10 bar parts
+    assert format_meter(
+        0, 1000, 13, ncols=20,
+        bar_format="************{bar:10}$$$$$$$$$$") == "************        "
     # n_cols allows for l_bar, bar, and some of r_bar
     # l_bar displays
     # bar displays
     # r_bar gets chopped
     # all 12 stars and 10 bar parts, but only 8/10 dollar signs
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=30,
-            bar_format="************{bar:10}$$$$$$$$$$") == \
-        "************          $$$$$$$$"
+    assert format_meter(
+        0, 1000, 13, ncols=30,
+        bar_format="************{bar:10}$$$$$$$$$$") == "************          $$$$$$$$"
     # trim left ANSI; escape is before trim zone
     # we only know it has ANSI codes, so we append an END code anyway
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=10,
-            bar_format="*****\033[22m****\033[0m***{bar:10}$$$$$$$$$$") == \
-        "*****\033[22m****\033[0m*\033[0m"
+    assert format_meter(
+        0, 1000, 13, ncols=10, bar_format="*****\033[22m****\033[0m***{bar:10}$$$$$$$$$$"
+    ) == "*****\033[22m****\033[0m*\033[0m"
     # trim left ANSI; escape is at trim zone
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=10,
-            bar_format="*****\033[22m*****\033[0m**{bar:10}$$$$$$$$$$") == \
-        "*****\033[22m*****\033[0m"
+    assert format_meter(
+        0, 1000, 13, ncols=10,
+        bar_format="*****\033[22m*****\033[0m**{bar:10}$$$$$$$$$$") == "*****\033[22m*****\033[0m"
     # trim left ANSI; escape is after trim zone
-    assert \
-        format_meter(
-            0, 1000, 13, ncols=10,
-            bar_format="*****\033[22m******\033[0m*{bar:10}$$$$$$$$$$") == \
-        "*****\033[22m*****\033[0m"
+    assert format_meter(
+        0, 1000, 13, ncols=10,
+        bar_format="*****\033[22m******\033[0m*{bar:10}$$$$$$$$$$") == "*****\033[22m*****\033[0m"
     # Check that bar_format correctly adapts {bar} size to the rest
-    assert format_meter(20, 100, 12, ncols=13, rate=8.1,
-                        bar_format=r'{l_bar}{bar}|{n_fmt}/{total_fmt}') == \
-        " 20%|" + unich(0x258f) + "|20/100"
-    assert format_meter(20, 100, 12, ncols=14, rate=8.1,
-                        bar_format=r'{l_bar}{bar}|{n_fmt}/{total_fmt}') == \
-        " 20%|" + unich(0x258d) + " |20/100"
+    assert format_meter(
+        20, 100, 12, ncols=13, rate=8.1,
+        bar_format=r'{l_bar}{bar}|{n_fmt}/{total_fmt}') == " 20%|" + unich(0x258f) + "|20/100"
+    assert format_meter(
+        20, 100, 12, ncols=14, rate=8.1,
+        bar_format=r'{l_bar}{bar}|{n_fmt}/{total_fmt}') == " 20%|" + unich(0x258d) + " |20/100"
     # Check wide characters
-    if sys.version_info >= (3,):
-        assert format_meter(0, 1000, 13, ncols=68, prefix='ｆｕｌｌｗｉｄｔｈ: ') == \
-            "ｆｕｌｌｗｉｄｔｈ:   0%|                  | 0/1000 [00:13<?, ?it/s]"
-        assert format_meter(0, 1000, 13, ncols=68, prefix='ニッポン [ﾆｯﾎﾟﾝ]: ') == \
-            "ニッポン [ﾆｯﾎﾟﾝ]:   0%|                    | 0/1000 [00:13<?, ?it/s]"
+    assert format_meter(0, 1000, 13, ncols=68, prefix='ｆｕｌｌｗｉｄｔｈ: ') == (
+        "ｆｕｌｌｗｉｄｔｈ:   0%|                  | 0/1000 [00:13<?, ?it/s]")
+    assert format_meter(0, 1000, 13, ncols=68, prefix='ニッポン [ﾆｯﾎﾟﾝ]: ') == (
+        "ニッポン [ﾆｯﾎﾟﾝ]:   0%|                    | 0/1000 [00:13<?, ?it/s]")
     # Check that bar_format can print only {bar} or just one side
     assert format_meter(20, 100, 12, ncols=2, rate=8.1,
-                        bar_format=r'{bar}') == \
-        unich(0x258d) + " "
+                        bar_format=r'{bar}') == unich(0x258d) + " "
     assert format_meter(20, 100, 12, ncols=7, rate=8.1,
-                        bar_format=r'{l_bar}{bar}') == \
-        " 20%|" + unich(0x258d) + " "
+                        bar_format=r'{l_bar}{bar}') == " 20%|" + unich(0x258d) + " "
     assert format_meter(20, 100, 12, ncols=6, rate=8.1,
-                        bar_format=r'{bar}|test') == \
-        unich(0x258f) + "|test"
+                        bar_format=r'{bar}|test') == unich(0x258f) + "|test"
 
 
 def test_ansi_escape_codes():
@@ -362,11 +317,11 @@ def test_si_format():
 
 def test_bar_formatspec():
     """Test Bar.__format__ spec"""
-    assert "{0:5a}".format(Bar(0.3)) == "#5   "
-    assert "{0:2}".format(Bar(0.5, charset=" .oO0")) == "0 "
-    assert "{0:2a}".format(Bar(0.5, charset=" .oO0")) == "# "
-    assert "{0:-6a}".format(Bar(0.5, 10)) == '##  '
-    assert "{0:2b}".format(Bar(0.5, 10)) == '  '
+    assert f"{Bar(0.3):5a}" == "#5   "
+    assert f"{Bar(0.5, charset=' .oO0'):2}" == "0 "
+    assert f"{Bar(0.5, charset=' .oO0'):2a}" == "# "
+    assert f"{Bar(0.5, 10):-6a}" == '##  '
+    assert f"{Bar(0.5, 10):2b}" == '  '
 
 
 def test_all_defaults():
@@ -435,7 +390,7 @@ def test_iterate_over_csv_rows():
     # Create a test csv pseudo file
     with closing(StringIO()) as test_csv_file:
         writer = csv.writer(test_csv_file)
-        for _ in _range(3):
+        for _ in range(3):
             writer.writerow(['test'] * 3)
         test_csv_file.seek(0)
 
@@ -449,7 +404,7 @@ def test_iterate_over_csv_rows():
 def test_file_output():
     """Test output to arbitrary file-like objects"""
     with closing(StringIO()) as our_file:
-        for i in tqdm(_range(3), file=our_file):
+        for i in tqdm(range(3), file=our_file):
             if i == 1:
                 our_file.seek(0)
                 assert '0/3' in our_file.read()
@@ -458,14 +413,14 @@ def test_file_output():
 def test_leave_option():
     """Test `leave=True` always prints info about the last iteration"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, leave=True):
+        for _ in tqdm(range(3), file=our_file, leave=True):
             pass
         res = our_file.getvalue()
         assert '| 3/3 ' in res
         assert '\n' == res[-1]  # not '\r'
 
     with closing(StringIO()) as our_file2:
-        for _ in tqdm(_range(3), file=our_file2, leave=False):
+        for _ in tqdm(range(3), file=our_file2, leave=False):
             pass
         assert '| 3/3 ' not in our_file2.getvalue()
 
@@ -486,7 +441,7 @@ def test_trange():
 def test_min_interval():
     """Test mininterval"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, mininterval=1e-10):
+        for _ in tqdm(range(3), file=our_file, mininterval=1e-10):
             pass
         assert "  0%|          | 0/3 [00:00<" in our_file.getvalue()
 
@@ -518,7 +473,7 @@ def test_max_interval():
             t.update(bigstep)
             t2.update(bigstep)
             # The next iterations should not trigger maxinterval (step 10)
-            for _ in _range(4):
+            for _ in range(4):
                 t.update(smallstep)
                 t2.update(smallstep)
                 timer.sleep(1e-5)
@@ -538,7 +493,7 @@ def test_max_interval():
             # Increase 10 iterations at once
             t.update(bigstep)
             # The next iterations should trigger maxinterval (step 5)
-            for _ in _range(4):
+            for _ in range(4):
                 t.update(smallstep)
                 timer.sleep(1e-2)
 
@@ -547,13 +502,12 @@ def test_max_interval():
     # Test iteration based tqdm with maxinterval effect
     timer = DiscreteTimer()
     with closing(StringIO()) as our_file:
-        with tqdm(_range(total), file=our_file, miniters=None,
+        with tqdm(range(total), file=our_file, miniters=None,
                   mininterval=1e-5, smoothing=1, maxinterval=1e-4) as t2:
             cpu_timify(t2, timer)
 
             for i in t2:
-                if i >= (bigstep - 1) and \
-                   ((i - (bigstep - 1)) % smallstep) == 0:
+                if i >= (bigstep - 1) and ((i - (bigstep - 1)) % smallstep) == 0:
                     timer.sleep(1e-2)
                 if i >= 3 * bigstep:
                     break
@@ -595,9 +549,9 @@ def test_max_interval():
     mininterval = 0.1
     maxinterval = 10
     with closing(StringIO()) as our_file:
-        t1 = tqdm(_range(total), file=our_file, miniters=None, smoothing=1,
+        t1 = tqdm(range(total), file=our_file, miniters=None, smoothing=1,
                   mininterval=mininterval, maxinterval=maxinterval)
-        t2 = tqdm(_range(total), file=our_file, miniters=None, smoothing=1,
+        t2 = tqdm(range(total), file=our_file, miniters=None, smoothing=1,
                   mininterval=0, maxinterval=maxinterval)
 
         cpu_timify(t1, timer1)
@@ -622,10 +576,25 @@ def test_max_interval():
         t2.close()
 
 
+def test_delay():
+    """Test delay"""
+    timer = DiscreteTimer()
+    with closing(StringIO()) as our_file:
+        t = tqdm(total=2, file=our_file, leave=True, delay=3)
+        cpu_timify(t, timer)
+        timer.sleep(2)
+        t.update(1)
+        assert not our_file.getvalue()
+        timer.sleep(2)
+        t.update(1)
+        assert our_file.getvalue()
+        t.close()
+
+
 def test_min_iters():
     """Test miniters"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, leave=True, mininterval=0, miniters=2):
+        for _ in tqdm(range(3), file=our_file, leave=True, mininterval=0, miniters=2):
             pass
 
         out = our_file.getvalue()
@@ -635,7 +604,7 @@ def test_min_iters():
         assert '| 3/3 ' in out
 
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, leave=True, mininterval=0, miniters=1):
+        for _ in tqdm(range(3), file=our_file, leave=True, mininterval=0, miniters=1):
             pass
 
         out = our_file.getvalue()
@@ -689,7 +658,7 @@ def test_dynamic_min_iters():
 
     # Check iterable based tqdm
     with closing(StringIO()) as our_file:
-        t = tqdm(_range(10), file=our_file, miniters=None, mininterval=None,
+        t = tqdm(range(10), file=our_file, miniters=None, mininterval=None,
                  smoothing=0.5)
         for _ in t:
             pass
@@ -697,7 +666,7 @@ def test_dynamic_min_iters():
 
     # No smoothing
     with closing(StringIO()) as our_file:
-        t = tqdm(_range(10), file=our_file, miniters=None, mininterval=None,
+        t = tqdm(range(10), file=our_file, miniters=None, mininterval=None,
                  smoothing=0)
         for _ in t:
             pass
@@ -705,7 +674,7 @@ def test_dynamic_min_iters():
 
     # No dynamic_miniters (miniters is fixed manually)
     with closing(StringIO()) as our_file:
-        t = tqdm(_range(10), file=our_file, miniters=1, mininterval=None)
+        t = tqdm(range(10), file=our_file, miniters=1, mininterval=None)
         for _ in t:
             pass
         assert not t.dynamic_miniters
@@ -714,12 +683,12 @@ def test_dynamic_min_iters():
 def test_big_min_interval():
     """Test large mininterval"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(2), file=our_file, mininterval=1E10):
+        for _ in tqdm(range(2), file=our_file, mininterval=1E10):
             pass
         assert '50%' not in our_file.getvalue()
 
     with closing(StringIO()) as our_file:
-        with tqdm(_range(2), file=our_file, mininterval=1E10) as t:
+        with tqdm(range(2), file=our_file, mininterval=1E10) as t:
             t.update()
             t.update()
             assert '50%' not in our_file.getvalue()
@@ -738,10 +707,10 @@ def test_smoothed_dynamic_min_iters():
             timer.sleep(1)
             t.update(10)
             # The next iterations should be partially skipped
-            for _ in _range(2):
+            for _ in range(2):
                 timer.sleep(1)
                 t.update(4)
-            for _ in _range(20):
+            for _ in range(20):
                 timer.sleep(1)
                 t.update()
 
@@ -770,7 +739,7 @@ def test_smoothed_dynamic_min_iters_with_min_interval():
 
             t.update(10)
             timer.sleep(1e-2)
-            for _ in _range(4):
+            for _ in range(4):
                 t.update()
                 timer.sleep(1e-2)
             out = our_file.getvalue()
@@ -778,7 +747,7 @@ def test_smoothed_dynamic_min_iters_with_min_interval():
 
     with closing(StringIO()) as our_file:
         # Test iteration-based tqdm
-        with tqdm(_range(total), file=our_file, miniters=None,
+        with tqdm(range(total), file=our_file, miniters=None,
                   mininterval=0.01, smoothing=1, maxinterval=0) as t2:
             cpu_timify(t2, timer)
 
@@ -837,7 +806,7 @@ def _rlock_creation_target():
 def test_disable():
     """Test disable"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, disable=True):
+        for _ in tqdm(range(3), file=our_file, disable=True):
             pass
         assert our_file.getvalue() == ''
 
@@ -851,19 +820,19 @@ def test_disable():
 def test_infinite_total():
     """Test treatment of infinite total"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, total=float("inf")):
+        for _ in tqdm(range(3), file=our_file, total=float("inf")):
             pass
 
 
 def test_nototal():
     """Test unknown total length"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm((i for i in range(10)), file=our_file, unit_scale=10):
+        for _ in tqdm(iter(range(10)), file=our_file, unit_scale=10):
             pass
         assert "100it" in our_file.getvalue()
 
     with closing(StringIO()) as our_file:
-        for _ in tqdm((i for i in range(10)), file=our_file,
+        for _ in tqdm(iter(range(10)), file=our_file,
                       bar_format="{l_bar}{bar}{r_bar}"):
             pass
         assert "10/?" in our_file.getvalue()
@@ -872,7 +841,7 @@ def test_nototal():
 def test_unit():
     """Test SI unit prefix"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), file=our_file, miniters=1, unit="bytes"):
+        for _ in tqdm(range(3), file=our_file, miniters=1, unit="bytes"):
             pass
         assert 'bytes/s' in our_file.getvalue()
 
@@ -886,7 +855,7 @@ def test_ascii():
 
     # Test ascii bar
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(3), total=15, file=our_file, miniters=1,
+        for _ in tqdm(range(3), total=15, file=our_file, miniters=1,
                       mininterval=0, ascii=True):
             pass
         res = our_file.getvalue().strip("\r").split("\r")
@@ -897,7 +866,7 @@ def test_ascii():
     # Test unicode bar
     with closing(UnicodeIO()) as our_file:
         with tqdm(total=15, file=our_file, ascii=False, mininterval=0) as t:
-            for _ in _range(3):
+            for _ in range(3):
                 t.update()
         res = our_file.getvalue().strip("\r").split("\r")
     assert u"7%|\u258b" in res[1]
@@ -907,7 +876,7 @@ def test_ascii():
     # Test custom bar
     for bars in [" .oO0", " #"]:
         with closing(StringIO()) as our_file:
-            for _ in tqdm(_range(len(bars) - 1), file=our_file, miniters=1,
+            for _ in tqdm(range(len(bars) - 1), file=our_file, miniters=1,
                           mininterval=0, ascii=bars, ncols=27):
                 pass
             res = our_file.getvalue().strip("\r").split("\r")
@@ -919,8 +888,7 @@ def test_update():
     """Test manual creation and updates"""
     res = None
     with closing(StringIO()) as our_file:
-        with tqdm(total=2, file=our_file, miniters=1, mininterval=0) \
-                as progressbar:
+        with tqdm(total=2, file=our_file, miniters=1, mininterval=0) as progressbar:
             assert len(progressbar) == 2
             progressbar.update(2)
             assert '| 2/2' in our_file.getvalue()
@@ -970,8 +938,7 @@ def test_close():
         res = our_file.getvalue()
         assert res[-1] == '\n'
         if not res.startswith(exres):
-            raise AssertionError("\n<<< Expected:\n{0}\n>>> Got:\n{1}\n===".format(
-                exres + ', ...it/s]\n', our_file.getvalue()))
+            raise AssertionError(f"\n<<< Expected:\n{exres}, ...it/s]\n>>> Got:\n{res}\n===")
 
     # Closing after the output stream has closed
     with closing(StringIO()) as our_file:
@@ -997,7 +964,7 @@ def test_smoothing():
 
     # -- Test disabling smoothing
     with closing(StringIO()) as our_file:
-        with tqdm(_range(3), file=our_file, smoothing=None, leave=True) as t:
+        with tqdm(range(3), file=our_file, smoothing=None, leave=True) as t:
             cpu_timify(t, timer)
 
             for _ in t:
@@ -1008,11 +975,11 @@ def test_smoothing():
     # 1st case: no smoothing (only use average)
     with closing(StringIO()) as our_file2:
         with closing(StringIO()) as our_file:
-            t = tqdm(_range(3), file=our_file2, smoothing=None, leave=True,
+            t = tqdm(range(3), file=our_file2, smoothing=None, leave=True,
                      miniters=1, mininterval=0)
             cpu_timify(t, timer)
 
-            with tqdm(_range(3), file=our_file, smoothing=None, leave=True,
+            with tqdm(range(3), file=our_file, smoothing=None, leave=True,
                       miniters=1, mininterval=0) as t2:
                 cpu_timify(t2, timer)
 
@@ -1038,11 +1005,11 @@ def test_smoothing():
     # 2nd case: use max smoothing (= instant rate)
     with closing(StringIO()) as our_file2:
         with closing(StringIO()) as our_file:
-            t = tqdm(_range(3), file=our_file2, smoothing=1, leave=True,
+            t = tqdm(range(3), file=our_file2, smoothing=1, leave=True,
                      miniters=1, mininterval=0)
             cpu_timify(t, timer)
 
-            with tqdm(_range(3), file=our_file, smoothing=1, leave=True,
+            with tqdm(range(3), file=our_file, smoothing=1, leave=True,
                       miniters=1, mininterval=0) as t2:
                 cpu_timify(t2, timer)
 
@@ -1061,11 +1028,11 @@ def test_smoothing():
     # 3rd case: use medium smoothing
     with closing(StringIO()) as our_file2:
         with closing(StringIO()) as our_file:
-            t = tqdm(_range(3), file=our_file2, smoothing=0.5, leave=True,
+            t = tqdm(range(3), file=our_file2, smoothing=0.5, leave=True,
                      miniters=1, mininterval=0)
             cpu_timify(t, timer)
 
-            t2 = tqdm(_range(3), file=our_file, smoothing=0.5, leave=True,
+            t2 = tqdm(range(3), file=our_file, smoothing=0.5, leave=True,
                       miniters=1, mininterval=0)
             cpu_timify(t2, timer)
 
@@ -1119,7 +1086,7 @@ def test_bar_format():
     with closing(StringIO()) as our_file:
         bar_format = r'hello world'
         with tqdm(ascii=False, bar_format=bar_format, file=our_file) as t:
-            assert isinstance(t.bar_format, _unicode)
+            assert isinstance(t.bar_format, str)
 
 
 def test_custom_format():
@@ -1148,7 +1115,7 @@ def test_eta(capsys):
                     bar_format='{l_bar}{eta:%Y-%m-%d}'):
         pass
     _, err = capsys.readouterr()
-    assert "\r100%|{eta:%Y-%m-%d}\n".format(eta=dt.now()) in err
+    assert f"\r100%|{dt.now():%Y-%m-%d}\n" in err
 
 
 def test_unpause():
@@ -1278,7 +1245,7 @@ def test_position():
     t1 = tqdm(desc='pos0 bar', position=0, **kwargs)
     t2 = tqdm(desc='pos1 bar', position=1, **kwargs)
     t3 = tqdm(desc='pos2 bar', position=2, **kwargs)
-    for _ in _range(2):
+    for _ in range(2):
         t1.update()
         t3.update()
         t2.update()
@@ -1368,9 +1335,10 @@ def test_deprecated_gui():
         try:
             t.update(1)
         except TqdmDeprecationWarning as e:
-            if ('Please use `tqdm.gui.tqdm(...)` instead of'
-                ' `tqdm(..., gui=True)`') \
-                    not in our_file.getvalue():
+            if (
+                'Please use `tqdm.gui.tqdm(...)` instead of `tqdm(..., gui=True)`'
+                not in our_file.getvalue()
+            ):
                 raise e
         else:
             raise DeprecationError('Should not allow manual gui=True without'
@@ -1380,14 +1348,15 @@ def test_deprecated_gui():
             # t.close()
             # len(tqdm._instances) += 1  # undo the close() decrement
 
-        t = tqdm(_range(3), gui=True, file=our_file, miniters=1, mininterval=0)
+        t = tqdm(range(3), gui=True, file=our_file, miniters=1, mininterval=0)
         try:
             for _ in t:
                 pass
         except TqdmDeprecationWarning as e:
-            if ('Please use `tqdm.gui.tqdm(...)` instead of'
-                ' `tqdm(..., gui=True)`') \
-                    not in our_file.getvalue():
+            if (
+                'Please use `tqdm.gui.tqdm(...)` instead of `tqdm(..., gui=True)`'
+                not in our_file.getvalue()
+            ):
                 raise e
         else:
             raise DeprecationError('Should not allow manual gui=True without'
@@ -1754,7 +1723,7 @@ def test_external_write():
 def test_unit_scale():
     """Test numeric `unit_scale`"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(9), unit_scale=9, file=our_file,
+        for _ in tqdm(range(9), unit_scale=9, file=our_file,
                       miniters=1, mininterval=0):
             pass
         out = our_file.getvalue()
@@ -1809,9 +1778,9 @@ def test_bool():
             assert not t
         with tqdm([0], **kwargs) as t:
             assert t
-        with tqdm((x for x in []), **kwargs) as t:
+        with tqdm(iter([]), **kwargs) as t:
             assert t
-        with tqdm((x for x in [1, 2, 3]), **kwargs) as t:
+        with tqdm(iter([1, 2, 3]), **kwargs) as t:
             assert t
         with tqdm(**kwargs) as t:
             try:
@@ -1956,7 +1925,7 @@ def test_screen_shape():
 def test_initial():
     """Test `initial`"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(9), initial=10, total=19, file=our_file,
+        for _ in tqdm(range(9), initial=10, total=19, file=our_file,
                       miniters=1, mininterval=0):
             pass
         out = our_file.getvalue()
@@ -1967,7 +1936,7 @@ def test_initial():
 def test_colour():
     """Test `colour`"""
     with closing(StringIO()) as our_file:
-        for _ in tqdm(_range(9), file=our_file, colour="#beefed"):
+        for _ in tqdm(range(9), file=our_file, colour="#beefed"):
             pass
         out = our_file.getvalue()
         assert '\x1b[38;2;%d;%d;%dm' % (0xbe, 0xef, 0xed) in out
@@ -1980,7 +1949,7 @@ def test_colour():
             assert "Unknown colour" in str(w[-1].message)
 
     with closing(StringIO()) as our_file2:
-        for _ in tqdm(_range(9), file=our_file2, colour="blue"):
+        for _ in tqdm(range(9), file=our_file2, colour="blue"):
             pass
         out = our_file2.getvalue()
         assert '\x1b[34m' in out
@@ -1992,3 +1961,24 @@ def test_closed():
         for i in trange(9, file=our_file, miniters=1, mininterval=0):
             if i == 5:
                 our_file.close()
+
+
+def test_reversed(capsys):
+    """Test reversed()"""
+    for _ in reversed(tqdm(range(9))):
+        pass
+    out, err = capsys.readouterr()
+    assert not out
+    assert '  0%' in err
+    assert '100%' in err
+
+
+def test_contains(capsys):
+    """Test __contains__ doesn't iterate"""
+    with tqdm(list(range(9))) as t:
+        assert 9 not in t
+        assert all(i in t for i in range(9))
+    out, err = capsys.readouterr()
+    assert not out
+    assert '  0%' in err
+    assert '100%' not in err
